// Raindrops.cpp
// Screensaver module for Blanket
// ©1997 Michael D. Crawford.  All Rights Reserved.
//  9 Nov 97 Michael Crawford crawford@scruznet.com http://www.scruznet.com/~crawford/

#include "SaverModule.h"
#include "BrowserLauncher.h"
#include "RainInfo.h"
#include "MyView.h"
#include "ConfigHandler.h"
#include "RainConfig.h"
#include "IntensityScroll.h"

RainInfo *gRainInfo;
ConfigHandler *gHandler;
RainConfig *gSettings;

int main( int argc, char **argv )
{
	return 0;
}

// ------------------------------------------------------------
// 		 module_initialize  [optional]
// ------------------------------------------------------------
// This function gets called once, immediately after saver has
// loaded your module
//
// It is not known at this point whether you are being loaded
// to save the screen or to configure yourself (or both)
//
// inSettings is a pointer to data inSettingsSize long that saver 
// has stored for you, it is your responsibility to free it before 
// your module is unloaded
//
// saver may pass you NULL and 0 if it cannot find any data that
// belongs to you

void
module_initialize(
	void	*inSettings,
	long	inSettingsSize)
{
	gSettings = new RainConfig;
	
	if ( inSettings ){
		if ( inSettingsSize != sizeof( RainConfig ) ){
			gSettings->intensity = 100;
		}else{
			gSettings->intensity = ((RainConfig*)inSettings)->intensity;
		}
	}else{
		gSettings->intensity = 100;
	}
	
	free( inSettings );
}

// ------------------------------------------------------------
// 		 module_cleanup  [optional]
// ------------------------------------------------------------
// This function gets called once, immediately before saver will
// unload your module
//
// This is the last chance to clean up after yourself
//
// Set outSettings to a pointer that is outSettingsSize long, 
// and saver will store it for you until the next time your module 
// is loaded
//
// Pass back NULL and 0 if you do not need to store anything
//
// Do not attempt to free what you pass back to saver

void
module_cleanup(
	void	**outSettings,
	long	*outSettingsSize)
{
	*outSettings = gSettings;
	*outSettingsSize = sizeof( RainConfig );
}

// ------------------------------------------------------------
// 		module_start_saving  [required]
// ------------------------------------------------------------
// This function gets called when it is time to "save" the screen
//
// You are given a BView in which you are to do your work
//
// The window in which the BView resides is not shown at this time,
// it is your responsibility to inView->Window()->Show() it
//
// Spawn your thread(s) here and return from this function ASAP
// Do all your drawing in inView, or in its children (if you 
// decide to add any)
//
// inView's View and Low color are set to black, High color is white
//
// saver never locks inView's window for you, so be sure to
// inView->Window()->Lock() before you try to do anything with
// inView, otherwise you win a free trip to the debugger

void
module_start_saving(
	BView	*inView)
{
	gRainInfo = new RainInfo( inView, gSettings->intensity );
	
	inView->Window()->Lock();
		
	inView->AddChild( gRainInfo->GetView() );

	inView->Window()->Show();	
	inView->Window()->Unlock();

	gRainInfo->StartThreads();
	
	return;	
}

// ------------------------------------------------------------
// 		 module_stop_saving  [required]
// ------------------------------------------------------------
// This function gets called right before the BWindow that
// you have been drawing into gets closed
//
// Exit from all your thread(s), free any memory that you no 
// longer need, unlock anything that you locked, be good and 
// clean up after yourself

void
module_stop_saving()
{

	gRainInfo->StopThreads();
	
	delete gRainInfo;
}

// ------------------------------------------------------------
// 		 module_start_config  [required]
// ------------------------------------------------------------
// This function gets called when the user has selected your 
// module in the settings panel
//
// You are free to add any children to inView or modify it
// in any way (but don't even think about deleting it)
//
// inView is 241 units wide and 244 units high

void
module_start_config(
	BView *inView)
{
	if (!inView->Window()->Lock())
		return;

	gHandler = new ConfigHandler;
	
	inView->Looper()->AddHandler( gHandler );
	
	BStringView *view = NULL;
	
	view = new BStringView(BRect(20.0, 20.0, 100.0, 36.0),
								 B_EMPTY_STRING,
								 "Raindrops 1.0.1");
	view->SetFont( be_bold_font );
	inView->AddChild(view);
	view->SetViewColor(inView->ViewColor());
	
	view = new BStringView(BRect(30.0, 40.0, 200.0, 52.0),
								 B_EMPTY_STRING,
								 "©1997-98 Michael D. Crawford");
	inView->AddChild(view);
	view->SetViewColor(inView->ViewColor());

	view = new BStringView(BRect(30.0, 60.0, 200.0, 72.0),
								 B_EMPTY_STRING,
								 "Crawford Software Consulting");
	inView->AddChild(view);
	view->SetViewColor(inView->ViewColor());
	
	view = new BStringView(BRect(30.0, 80.0, 200.0, 92.0),
								 B_EMPTY_STRING,
								 "crawford@scruznet.com");
	inView->AddChild(view);
	view->SetViewColor(inView->ViewColor());
	
	BMessage *msg = new BMessage( BrowserLauncher::OpenURL );
	msg->AddString( "URL", "http://www.scruznet.com/~crawford/Computers/be.html" );
	
	BButton *but = new BButton(BRect(30.0, 100.0, 200.0, 120.0),
								 "SourceButton",
								 "Get Source Code",
								 msg );
	but->SetTarget( gHandler );
	
	inView->AddChild(but);

	msg = new BMessage( BrowserLauncher::OpenURL );
	msg->AddString( "URL", "http://www.scruznet.com/~crawford/crawford.html" );
	
	but = new BButton(BRect(5.0, 130.0, 225.0, 150.0),
								 "CSCButton",
								 "Crawford Software Consulting Web Page",
								 msg );
	but->SetTarget( gHandler );
	
	inView->AddChild(but);
	
	BScrollBar *scroll = new IntensityScroll( BRect( 10, 180, 230, 180 + B_H_SCROLL_BAR_HEIGHT ),
											"Intensity" );	
	inView->AddChild( scroll );
	
	BStringView *s = new BStringView( BRect( 80, 160, 200, 175 ),
										"IntensityLabel",
										"Rainfall Intensity" );
	inView->AddChild( s );
	
	s = new BStringView( BRect( 10, 204, 60, 219 ),
							"SoftLabel",
							"Soft" );
	inView->AddChild( s );
	
	s = new BStringView( BRect( 200, 204, 230, 219 ),
							"HardLabel",
							"Hard" );
	inView->AddChild( s );
	
	inView->Window()->Unlock();

}


// ------------------------------------------------------------
// 		 module_stop_config  [required]
// ------------------------------------------------------------
// This function gets called right before the BView that was passed
// to you in module_start_config() gets deleted
//
// Free any memory that you may have allocated, you know the game...
//
// Since your view will be deleted, it is not necessary to delete 
// the children that you added to it

void
module_stop_config()
{
	BLooper *looper = gHandler->Looper();
	looper->Lock();
		
	looper->RemoveHandler( gHandler );
	
	looper->Unlock();
	delete gHandler;
	
	return;
}




